//Sweet Math Quiz
//Use LCD keypad shield.
//Connect servo on pin A1
//To be used with LEGO candy distributor
//Building instructions on https://www.instructables.com/Sweet-Math-Quiz/
//
//R. Oldeman Nov 2020, CC BY-NC-SA 4.0 licence
#include <LiquidCrystal.h>
#include <Servo.h>

#define pos_in    0 //servo angle for tray inside
#define pos_out 180 //servo angle for tray outside

#define startlevel 1
#define maxtime   10
#define nquest     5
#define mingood    5

#define btnpin   A0
#define servopin A1

#define btnRIGHT  0
#define btnUP     1
#define btnDOWN   2
#define btnLEFT   3
#define btnSELECT 4
#define btnNONE   5

// initialize the library with pins RS,EN,D4,D5,D6,D7
// these values correspond o the standard LCD keypad shield
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

Servo myservo;

void setup()
{
  pinMode(btnpin, INPUT);
  lcd.begin(16, 2);
  myservo.attach(servopin);
  myservo.write(pos_in);
}


void loop() {
  char line[20];
  char question[20];
  char answer_right[10];
  char answer_wrong[10];
  char question_full[30];
  int level = startlevel;
  int ngood = 0;
  int iquest = 0;

  lcd.clear();
  lcd.setCursor(0, 0); lcd.print("Sweet Math Quiz");
  lcd.setCursor(0, 1); lcd.print("L=start R=help");

  int btn;

  //initialize
  while (true) {
    btn = read_LCD_buttons();
    if (btn != btnNONE)break;
  }
  if (btn == btnRIGHT) {
    lcd.clear();
    lcd.setCursor(0, 0); lcd.print("LEFT if FALSE");
    lcd.setCursor(0, 1); lcd.print("RIGHT if TRUE");
    while (read_LCD_buttons() != btnNONE) {}
    while (read_LCD_buttons() == btnNONE) {}
    lcd.clear();
    sprintf(line, "get %d/%d good", nquest, mingood);
    lcd.setCursor(0, 0); lcd.print(line);
    lcd.setCursor(0, 1); lcd.print("press to start");
    while (read_LCD_buttons() != btnNONE) {}
    while (read_LCD_buttons() == btnNONE) {}
  }

  randomSeed(millis());

  //main loop
  while (true) {
    while (read_LCD_buttons() != btnNONE) {}

    //display level and number of questions
    if (false && iquest == 0) {
      lcd.clear();
      sprintf(line, "level %d", level);
      lcd.setCursor(0, 0); lcd.print(line);
      sprintf(line, "%d questions", nquest);
      lcd.setCursor(0, 1); lcd.print(line);
      while (read_LCD_buttons() == btnNONE) {}
      while (read_LCD_buttons() != btnNONE) {}
    }

    bool rightanswer = false;
    bool rightquestion = random(2);
    mathquestion(question, answer_right, answer_wrong, level);
    if (rightquestion)sprintf(question_full, "%s=%s?", question, answer_right);
    if (!rightquestion)sprintf(question_full, "%s=%s?", question, answer_wrong);
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print(question_full);
    unsigned long t0 = millis();
    int timeleft = maxtime - (millis() - t0) / 1000;
    lcd.setCursor(0, 1); lcd.print(timeleft);
    while (true) {
      int newtimeleft = maxtime - (millis() - t0) / 1000;
      if (newtimeleft < timeleft) {
        timeleft = newtimeleft;
        lcd.setCursor(0, 1); lcd.print("                ");
        lcd.setCursor(0, 1); lcd.print(timeleft);
      }
      if (timeleft == 0)break;
      btn = read_LCD_buttons();
      if ((btn == btnRIGHT && rightquestion) || (btn == btnLEFT && !rightquestion) ) {
        rightanswer = true;
        break;
      }
      if ((btn == btnRIGHT && !rightquestion) || (btn == btnLEFT && rightquestion) ) {
        rightanswer = false;
        break;
      }
    }
    if (rightanswer) {
      lcd.clear();
      lcd.setCursor(0, 0); lcd.print(question_full);
      if (rightquestion) {
        lcd.setCursor(0, 1);
        lcd.print("YES it is true!");
      }
      if (!rightquestion) {
        lcd.setCursor(0, 1);
        lcd.print("YES it is false!");
      }
      ngood += 1;
    }
    if (!rightanswer) {
      if (rightquestion) {
        lcd.setCursor(0, 1);
        lcd.print("NO it is true!");
      }
      if (!rightquestion) {
        lcd.setCursor(0, 1);
        lcd.print("NO it is false!");
      }
    }
    iquest += 1;
    delay(1000);
    if (iquest == nquest) {
      lcd.clear();
      sprintf(line, "score %d/%d", ngood, nquest);
      lcd.setCursor(0, 0); lcd.print(line);
      if (ngood >= mingood) {
        level += 1;
        sprintf(line, "PASSED->LEVEL%d", level);
        lcd.setCursor(0, 1); lcd.print(line);
        givecandy();
      }
      if (ngood < mingood) {
        if (level > 0)level -= 1;
        sprintf(line, "FAILED->LEVEL%d", level);
        lcd.setCursor(0, 1); lcd.print(line);
      }
      while (read_LCD_buttons() != btnNONE) {}
      while (read_LCD_buttons() == btnNONE) {}
      iquest = 0; ngood = 0;


    }


  }
}

//activate the servo motor to distribute candies
void givecandy() {
  //open
  for (int pos = pos_in; pos <= pos_out; pos++) {
    myservo.write(pos);
    delay(5);
  }
  //wait
  delay(1000);
  //close
  for (int pos = pos_out; pos >= pos_in; pos--) {
    myservo.write(pos);
    delay(5);
  }
}


// generate a question with a right and wrong answer
void mathquestion(char*q, char *a_right, char *a_wrong, int level) {
  int op = random(4);
  int maxval = 10 * level;
  if (op > 1) maxval = 3 * level;
  int a = random(maxval) + 1;
  int b = random(maxval);
  int b2 = random(maxval);
  if (b2 == b) b2 = b + 1;
  switch (op) {
    case 0:
      sprintf(q, "%d+%d", a, b);
      sprintf(a_right, "%d", a + b);
      sprintf(a_wrong, "%d", a + b2);
      break;
    case 1:
      sprintf(q, "%d-%d", a + b, a);
      sprintf(a_right, "%d", b);
      sprintf(a_wrong, "%d", b2);
      break;
    case 2:
      sprintf(q, "%dx%d", a, b);
      sprintf(a_right, "%d", a * b);
      sprintf(a_wrong, "%d", a * b2);
      break;
    case 3:
      sprintf(q, "%d/%d", a * b, a);
      sprintf(a_right, "%d", b);
      sprintf(a_wrong, "%d", b2);
      break;
  }
}

// read the buttons
int read_LCD_buttons() {
  int val = analogRead(btnpin);
  if (val > 1000) return btnNONE;
  if (val <   50) return btnRIGHT;
  if (val <  250) return btnUP;
  if (val <  450) return btnDOWN;
  if (val <  650) return btnLEFT;
  if (val <  850) return btnSELECT;
  return btnNONE;
}
